#ifndef _SWD_COMMS_PACKET_H_
#define _SWD_COMMS_PACKET_H_


#include "../ModulaR/commsPacket.h"


#define NUM_REGS						16

#define SWD_COMMS_CMD_VER_INFO		0x00	// () -> SwdCommsVerInfoRespPacketV*
#define SWD_COMMS_CMD_ATTACH		0x01	// () -> SwdCommsAttachRespPacket
#define SWD_COMMS_CMD_MEM_READ		0x02	// SwdCommsMemPacket -> SwdCommsMemPacket
#define SWD_COMMS_CMD_MEM_WRITE		0x03	// SwdCommsMemPacket -> SwdCommsMemPacket
#define SWD_COMMS_CMD_REGS_READ		0x04	// () -> SwdCommsRegsPacket
#define SWD_COMMS_CMD_REGS_WRITE	0x05	// SwdCommsRegsPacket -> ()
#define SWD_COMMS_CMD_GO			0x06	// () -> (u8 success)
#define SWD_COMMS_CMD_RESET			0x07	// () -> (u8 success)
#define SWD_COMMS_CMD_STOP			0x08	// () -> (u8 CORTEX_W_* mask or CORTEX_W_FAIL)
#define SWD_COMMS_CMD_IS_STOPPED	0x09	// () -> (u8 CORTEX_W_* mask or CORTEX_W_FAIL)
#define SWD_COMMS_CMD_SINGLE_STEP	0x0A	// () -> (u8 CORTEX_W_* mask or CORTEX_W_FAIL)

#define SWD_COMMS_SWD_WIRE_BUS_R	0x3E	// SwdCommsWireBusPacket -> SwdCommsWireBusPacket
#define SWD_COMMS_SWD_WIRE_BUS_W	0x3F	// SwdCommsWireBusPacket -> SwdCommsWireBusPacket
#define SWD_TRACE_LOG_READ			0x40	// (u32) -> (u8[])
#define SWD_POWER_CTRL				0x41	// simple control: (u8 on) -> (u8 success)  === OR === variable-voltage control: (u16 millivolts) -> (u8 success)
#define SWD_COMMS_CMD_SET_CLOCK		0x42	// (u32 clock) -> (u32 clock)

#pragma pack(push,1)
struct SwdCommsWireBusPacket {
	uint8_t returnVal;		//SWD return val, ignored on input
	uint8_t ap;
	uint8_t a23;
	uint32_t val;
};
#pragma pack(pop)

//EACH ONE OF THESE MUST!!! include the previous's bytes up front verbatim
#pragma pack(push,1)
struct SwdCommsVerInfoRespPacketV1 {
	uint32_t swdAppVer;
};
#pragma pack(pop)

#pragma pack(push,1)
struct SwdCommsVerInfoRespPacketV2 {
	uint32_t swdAppVer;
	uint32_t flags;
	uint32_t maxXferBytes;		//SWD_COMMS_MAX_XFER_BYTES
};
#pragma pack(pop)

#pragma pack(push,1)
struct SwdCommsVerInfoRespPacketV3 {
	uint32_t swdAppVer;
	uint32_t flags;
	uint32_t maxXferBytes;		//SWD_COMMS_MAX_XFER_BYTES
	uint8_t hwType;				//SWD_COMMS_HW_TYP_*
	uint8_t hwVer;				//SWD_COMMS_HW_*_VER_*
};
#pragma pack(pop)

#pragma pack(push,1)
struct SwdCommsVerInfoRespPacketV4 {
	uint32_t swdAppVer;
	uint32_t flags;
	uint32_t maxXferBytes;		//SWD_COMMS_MAX_XFER_BYTES
	uint8_t hwType;				//SWD_COMMS_HW_TYP_*
	uint8_t hwVer;				//SWD_COMMS_HW_*_VER_*
	uint16_t millivoltsMin;		//for power suply ability (zeroes if not supported) only usable if flags has PWR_FLAG_PWR_CTRL_SETTABLE or PWR_FLAG_PWR_CTRL_ON_OFF bit set!
	uint16_t millivoltsMax;		//in case of PWR_FLAG_PWR_CTRL_ON_OFF, should be same as above
	uint16_t milliampsMax;		//max supply current
};
#pragma pack(pop)

#pragma pack(push,1)
struct SwdCommsVerInfoRespPacketV5 {
	uint32_t swdAppVer;
	uint32_t flags;
	uint32_t maxXferBytes;		//SWD_COMMS_MAX_XFER_BYTES
	uint8_t hwType;				//SWD_COMMS_HW_TYP_*
	uint8_t hwVer;				//SWD_COMMS_HW_*_VER_*
	uint16_t millivoltsMin;		//for power suply ability (zeroes if not supported) only usable if flags has PWR_FLAG_PWR_CTRL_SETTABLE or PWR_FLAG_PWR_CTRL_ON_OFF bit set!
	uint16_t millivoltsMax;		//in case of PWR_FLAG_PWR_CTRL_ON_OFF, should be same as above
	uint16_t milliampsMax;		//max supply current
	uint32_t maxClockRate;		//in Hz
};
#pragma pack(pop)


#define USB_FLAGS_FTR_OUT_SUPPORTED		0x00000001UL
#define USB_FLAGS_NEED_PACKET_PADDING	0x00000002UL
#define PWR_FLAG_PWR_CTRL_ON_OFF		0x00000004UL		//old-style power control (on or off 3.3V supply, 50mA max)
#define UART_FLAG_UART_EXISTS			0x00000008UL
#define PWR_FLAG_PWR_CTRL_SETTABLE		0x00000010UL		//new-style power control (300mA, settable voltage)
#define SWD_FLAG_CLOCK_SPEED_SETTABLE	0x00000020UL		//if set, maxClockRate in SwdCommsVerInfoRespPacket is usable

#define SWD_COMMS_HW_TYP_UNKNOWN		0xFF
#define SWD_COMMS_HW_TYP_AVR_PROTO		0x00

#define SWD_FLAG_HAS_FPU		0x01

#define ERR_FLAG_TYPE_MASK		0xF0
#define ERR_FLAG_TYPE_SWD		0x00
#define ERR_FLAG_TYPE_MEMAP		0x10
#define ERR_FLAG_TYPE_CORTEX	0x20

#pragma pack(push,1)
struct SwdCommsAttachRespPacket {
	uint16_t cortexType;		//if zero, we have error, else we have flags
	union {
		uint8_t flags;
		uint8_t error;
	};
	uint32_t romTableBase;
	uint32_t targetid;	//DAPv2 only, else 0
};
#pragma pack(pop)

#define SWD_MEM_NUM_WORDS_ERROR				0xFFFF
#define SWD_COMMS_MAX_XFER_WORDS_NO_ACK		0xFFFE	/* write SWD_COMMS_MAX_XFER_WORDS and send no ack */
#pragma pack(push,1)
struct SwdCommsMemPacket {
	uint32_t addr;
	uint16_t numWords;	//up to SWD_COMMS_MAX_XFER_WORDS or SWD_MEM_NUM_WORDS_ERROR on error
	uint32_t words[];	//numWords elements for write req or read resp, else nonexistent
};
#pragma pack(pop)

#define SWD_COMMS_REG_SET_BASE	0x00	//R0..R15
#define SWD_COMMS_REG_SET_CTRL	0x01	//XPSR, MSP, PSP, CFBP, FPCSR
#define SWD_COMMS_REG_SET_FP0	0x02	//S0..S15	//this and the next must be contiguous
#define SWD_COMMS_REG_SET_FP1	0x03	//S16..S31	//this and the prev must be contiguous
#define SWD_COMMS_REG_SET_ERROR	0xFF

#define SWD_REGS_NUM_Rx(x)		(NUM_REGS * SWD_COMMS_REG_SET_BASE + x)
#define SWD_REGS_NUM_XPSR		(NUM_REGS * SWD_COMMS_REG_SET_CTRL + 0)
#define SWD_REGS_NUM_MSP		(NUM_REGS * SWD_COMMS_REG_SET_CTRL + 1)
#define SWD_REGS_NUM_PSP		(NUM_REGS * SWD_COMMS_REG_SET_CTRL + 2)
#define SWD_REGS_NUM_CFBP		(NUM_REGS * SWD_COMMS_REG_SET_CTRL + 3)
#define SWD_REGS_NUM_FPCSR		(NUM_REGS * SWD_COMMS_REG_SET_CTRL + 4)
#define SWD_REGS_NUM_Sx(x)		(NUM_REGS * SWD_COMMS_REG_SET_FP0 + (x))

#pragma pack(push,1)
struct SwdCommsRegsPacket {
	uint8_t regSet;		//or SWD_COMMS_REG_SET_ERROR
	uint32_t regs[];	//NUM_REGS or missing
};
#pragma pack(pop)

#endif
